/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/

package net.sf.intelliplugin.visualgc.action;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.application.ApplicationManager;
import net.sf.intelliplugin.visualgc.model.GCSample;
import net.sf.intelliplugin.visualgc.util.DialogHelper;
import net.sf.intelliplugin.visualgc.ui.VMInfoPanel;
import net.sf.intelliplugin.visualgc.ui.IntelliHeapComponent;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import sun.jvmstat.monitor.VmIdentifier;

/**
 * Action class for displaying VM information when connected to a JVM.
 * 
 * @author Bart Cremers
 * @since 3-dec-2007
 */
public class VMInfoAction extends AnAction {

    public void actionPerformed(AnActionEvent e) {
        IntelliHeapComponent component = (IntelliHeapComponent) ApplicationManager.getApplication().getComponent(
            IntelliHeapComponent.COMPONENT_NAME);
        if (component != null) {
            VMInfoPanel infoPanel = initializeInfoPanel(component.getCurrentSample());
            VmIdentifier vmIdentifier = component.getMonitoredvm().getVmIdentifier();
            final UpdaterThread thread = new UpdaterThread(component, infoPanel);
            thread.start();

            DialogHelper.showInDialog("Visual GC - " + vmIdentifier + " - VM Info",
                                      infoPanel, getButtonPane(thread),
                                      new WindowAdapter() {
                                          public void windowClosing(WindowEvent e) {
                                              thread.finish();
                                          }
                                      });
        }
    }

    private JPanel getButtonPane(final UpdaterThread updaterThread) {
        JPanel buttons = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        buttons.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

        JButton okButton = new JButton("    OK    ");
        buttons.add(okButton);

        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                updaterThread.finish();
                SwingUtilities.getWindowAncestor((Component) e.getSource()).dispose();
            }
        });
        return buttons;
    }

    private VMInfoPanel initializeInfoPanel(GCSample gcsample) {
        return new VMInfoPanel(gcsample);
    }

    /**
     * Thread used to update the JVM information in real time.
     */
    private static class UpdaterThread extends Thread {

        private final IntelliHeapComponent intelliHeapComponent;
        private final VMInfoPanel vmInfoPanel;

        private volatile boolean run = true;

        private UpdaterThread(IntelliHeapComponent intelliHeapComponent, VMInfoPanel vmInfoPanel) {
            this.intelliHeapComponent = intelliHeapComponent;
            this.vmInfoPanel = vmInfoPanel;
            setDaemon(true);
        }

        public void run() {
            while (run) {
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    Thread.interrupted();
                }
                vmInfoPanel.setSample(intelliHeapComponent.getCurrentSample());
            }
        }

        public void finish() {
            run = false;
        }
    }
}
