/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/
package net.sf.intelliplugin.visualgc.config;

import com.intellij.ui.ColorPanel;
import net.sf.intelliplugin.visualgc.util.GridBagUtil;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * Configuration panel implementation.
 * 
 * @author Bart Cremers
 * @since 3-dec-2007
 */
public class ConfigurationPanel extends JPanel implements ActionListener {

    private final IntelliHeapConfiguration configuration;

    private static final String[] LABELS = new String[] {
        "Background Color:", "Primary Grid Color:", "Secundary Grid Color:", "Border Color:",
        "Survivor Space Color:", "Eden Space Color:", "Tenured Space Color:", "Perm Space Color:",
        "Hotspot Time Color:", "Classload Time Color:", "Garbage Collection Time Color:", "Finalizer Time Color:"
    };

    public ConfigurationPanel(IntelliHeapConfiguration configuration) {
        super(new GridBagLayout());
        this.configuration = configuration;

        ColorPanel cpBackgroundColor = new ColorPanel();
        cpBackgroundColor.addActionListener(this);
        cpBackgroundColor.setActionCommand(IntelliHeapConfiguration.BACKGROUND_COLOR);
        cpBackgroundColor.setSelectedColor(configuration.getBackgroundColor());

        ColorPanel cpPrimaryGridColor = new ColorPanel();
        cpPrimaryGridColor.addActionListener(this);
        cpPrimaryGridColor.setActionCommand(IntelliHeapConfiguration.PRIMARY_GRID_COLOR);
        cpPrimaryGridColor.setSelectedColor(configuration.getPrimaryGridColor());

        ColorPanel cpSecundaryGridColor = new ColorPanel();
        cpSecundaryGridColor.addActionListener(this);
        cpSecundaryGridColor.setActionCommand(IntelliHeapConfiguration.SECUNDARY_GRID_COLOR);
        cpSecundaryGridColor.setSelectedColor(configuration.getSecundaryGridColor());

        ColorPanel cpBorderColor = new ColorPanel();
        cpBorderColor.addActionListener(this);
        cpBorderColor.setActionCommand(IntelliHeapConfiguration.BORDER_COLOR);
        cpBorderColor.setSelectedColor(configuration.getBorderColor());

        ColorPanel cpSurvivorColor = new ColorPanel();
        cpSurvivorColor.addActionListener(this);
        cpSurvivorColor.setActionCommand(IntelliHeapConfiguration.SURVIVOR_COLOR);
        cpSurvivorColor.setSelectedColor(configuration.getSurvivorColor());

        ColorPanel cpEdenColor = new ColorPanel();
        cpEdenColor.addActionListener(this);
        cpEdenColor.setActionCommand(IntelliHeapConfiguration.EDEN_COLOR);
        cpEdenColor.setSelectedColor(configuration.getEdenColor());

        ColorPanel cpTenuredColor = new ColorPanel();
        cpTenuredColor.addActionListener(this);
        cpTenuredColor.setActionCommand(IntelliHeapConfiguration.TENURED_COLOR);
        cpTenuredColor.setSelectedColor(configuration.getTenuredColor());

        ColorPanel cpPermColor = new ColorPanel();
        cpPermColor.addActionListener(this);
        cpPermColor.setActionCommand(IntelliHeapConfiguration.PERM_COLOR);
        cpPermColor.setSelectedColor(configuration.getPermColor());

        ColorPanel cpHotspotColor = new ColorPanel();
        cpHotspotColor.addActionListener(this);
        cpHotspotColor.setActionCommand(IntelliHeapConfiguration.HOTSPOT_COLOR);
        cpHotspotColor.setSelectedColor(configuration.getHotspotColor());

        ColorPanel cpLoaderColor = new ColorPanel();
        cpLoaderColor.addActionListener(this);
        cpLoaderColor.setActionCommand(IntelliHeapConfiguration.LOADER_COLOR);
        cpLoaderColor.setSelectedColor(configuration.getLoaderColor());

        ColorPanel cpGCColor = new ColorPanel();
        cpGCColor.addActionListener(this);
        cpGCColor.setActionCommand(IntelliHeapConfiguration.GC_COLOR);
        cpGCColor.setSelectedColor(configuration.getGcColor());

        ColorPanel cpFinalizerColor = new ColorPanel();
        cpFinalizerColor.addActionListener(this);
        cpFinalizerColor.setActionCommand(IntelliHeapConfiguration.FINALIZER_COLOR);
        cpFinalizerColor.setSelectedColor(configuration.getFinalizerColor());

        JComponent[] jComponents = {
            cpBackgroundColor, null, cpPrimaryGridColor, cpSecundaryGridColor, null, cpBorderColor, null,
            cpSurvivorColor, cpEdenColor, cpTenuredColor, cpPermColor, null, cpHotspotColor, cpLoaderColor, cpGCColor,
            cpFinalizerColor
        };

        GridBagUtil.createVerticalForm(this, LABELS, jComponents);
    }

    public void actionPerformed(ActionEvent e) {
        ColorPanel cp = (ColorPanel) e.getSource();
        Color newColor = cp.getSelectedColor();
        configuration.setValue(cp.getActionCommand(), newColor);
    }
}
