/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/
package net.sf.intelliplugin.visualgc.model;

/**
 * @author Bart Cremers
 * @since 20-nov-2007
 */
public interface Model {

    /**
     * @return the minimum size of the new generation
     */
    long getNewGenMinSize();

    /**
     * @return the maximum size of the new generation
     */
    long getNewGenMaxSize();

    /**
     * @return the current size of the new generation
     */
    long getNewGenCurSize();

    /**
     * @return the current amount of the new generation used for eden
     */
    long getEdenUsed();

    /**
     * @return the current amount of the new generation used for the "from" space (survivor space 0)
     */
    long getSurvivor0Used();

    /**
     * @return the current amount of the new generation used for the "to" space (survivor space 1)
     */
    long getSurvivor1Used();

    /**
     * @return the current amount of the old generation used
     */
    long getTenuredUsed();

    /**
     * @return the current amount of the "perm" space used
     */
    long getPermUsed();

    /**
     * @return the current eden size
     */
    long getEdenSize();

    /**
     * @return the current "from" space size
     */
    long getSurvivor0Size();

    /**
     * @return the current "to" space size
     */
    long getSurvivor1Size();

    /**
     * @return the current old generation size
     */
    long getTenuredSize();

    /**
     * @return the current "perm" space size
     */
    long getPermSize();

    /**
     * @return the capacity of eden
     */
    long getEdenCapacity();

    /**
     * @return the capacity of the "from" space
     */
    long getSurvivor0Capacity();

    /**
     * @return the capacity of the "to" space
     */
    long getSurvivor1Capacity();

    /**
     * @return the capacity of the old generation space
     */
    long getTenuredCapacity();

    /**
     * @return the capacity of the "perm" space
     */
    long getPermCapacity();

    /**
     * @return the amount of minor garbage collections
     */
    long getEdenGCEvents();

    /**
     * @return the amount of major garbage collections
     */
    long getTenuredGCEvents();

    /**
     * @return the total amount of time used for minor collections (in ms)
     */
    long getEdenGCTime();

    /**
     * @return the total amount of time used for major collections (in ms)
     */
    long getTenuredGCTime();

    /**
     * @return the threshold for the old generation
     */
    long getTenuringThreshold();

    /**
     * @return the maximum threshold for the old generation
     */
    long getMaxTenuringThreshold();

    /**
     * @return the desired survivor size
     */
    long getDesiredSurvivorSize();

    /**
     * @return the size of the age table
     */
    long getAgeTableSize();

    /**
     * @return the last cause for garbage collection
     */
    String getLastGCCause();

    /**
     * @return the current cause for garbage collection
     */
    String getCurrentGCCause();

    /**
     * @return the sizes of the different age tables
     */
    long[] getAgeTableSizes();

    /**
     * @param al
     */
    void getAgeTableSizes(long[] al);

    /**
     * @return the total time spent in classloading
     */
    long getClassLoadTime();

    /**
     * @return the amount of classes loaded
     */
    long getClassesLoaded();

    /**
     * @return the amount of classes unloaded
     */
    long getClassesUnloaded();

    /**
     * @return the amount of bytes loaded for classloading
     */
    long getClassBytesLoaded();

    /**
     * @return the amount of bytes unloaded for class-unloading
     */
    long getClassBytesUnloaded();

    /**
     * @return the amount of time the hotspot compiler spent in compiling
     */
    long getTotalCompileTime();

    /**
     * @return
     */
    long getTotalCompile();

    void initializeFinalizer();

    boolean isFinalizerInitialized();

    long getFinalizerTime();

    long getFinalizerCount();

    long getFinalizerQLength();

    long getFinalizerQMaxLength();

    long getOsElapsedTime();

    long getOsFrequency();

    String getJavaCommand();

    String getJavaHome();

    String getVmArgs();

    String getVmFlags();

    String getClassPath();

    String getEndorsedDirs();

    String getExtDirs();

    String getLibraryPath();

    String getBootClassPath();

    String getBootLibraryPath();

    String getVmInfo();

    String getVmName();

    String getVmVersion();

    String getVmVendor();

    String getVmSpecName();

    String getVmSpecVersion();

    String getVmSpecVendor();

    long getLastModificationTime();
}