/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/
package net.sf.intelliplugin.visualgc.model;

import sun.jvmstat.monitor.LongMonitor;
import sun.jvmstat.monitor.MonitorException;
import sun.jvmstat.monitor.MonitoredVm;
import sun.jvmstat.monitor.StringMonitor;

/**
 * @author Bart Cremers
 * @since 20-nov-2007
 */

class MonitoredVmModel implements Model, Constants {

    private final MonitoredVm vm;
    private LongMonitor newGenMinSize;
    private LongMonitor newGenMaxSize;
    private LongMonitor newGenCurSize;
    private LongMonitor edenSize;
    private LongMonitor edenCapacity;
    private LongMonitor edenUsed;
    private LongMonitor edenGCTime;
    private LongMonitor edenGCEvents;
    private LongMonitor survivor0Size;
    private LongMonitor survivor0Capacity;
    private LongMonitor survivor0Used;
    private LongMonitor survivor1Size;
    private LongMonitor survivor1Capacity;
    private LongMonitor survivor1Used;
    private LongMonitor tenuredSize;
    private LongMonitor tenuredCapacity;
    private LongMonitor tenuredUsed;
    private LongMonitor tenuredGCTime;
    private LongMonitor tenuredGCEvents;
    private LongMonitor permSize;
    private LongMonitor permCapacity;
    private LongMonitor permUsed;
    private LongMonitor tenuringThreshold;
    private LongMonitor maxTenuringThreshold;
    private LongMonitor desiredSurvivorSize;
    private LongMonitor ageTableSize;
    private LongMonitor[] ageTableSizes;
    private StringMonitor lastGCCause;
    private StringMonitor currentGCCause;
    private StringMonitor collectorName;
    private boolean finalizerInitialized;
    private LongMonitor finalizerTime;
    private LongMonitor finalizerQLength;
    private LongMonitor finalizerQMaxLength;
    private LongMonitor finalizerCount;
    private LongMonitor classLoadTime;
    private LongMonitor classesLoaded;
    private LongMonitor classesUnloaded;
    private LongMonitor classBytesLoaded;
    private LongMonitor classBytesUnloaded;
    private LongMonitor totalCompileTime;
    private LongMonitor totalCompile;
    private LongMonitor osElapsedTime;
    private LongMonitor osFrequency;
    private StringMonitor javaCommand;
    private StringMonitor javaHome;
    private StringMonitor vmArgs;
    private StringMonitor vmFlags;
    private StringMonitor vmInfo;
    private StringMonitor vmName;
    private StringMonitor vmVersion;
    private StringMonitor vmVendor;
    private StringMonitor vmSpecName;
    private StringMonitor vmSpecVersion;
    private StringMonitor vmSpecVendor;
    private StringMonitor classPath;
    private StringMonitor bootClassPath;
    private StringMonitor libraryPath;
    private StringMonitor bootLibraryPath;
    private StringMonitor endorsedDirs;
    private StringMonitor extDirs;
    private LongMonitor lastModificationTime;

    synchronized void initialize_finalizer() {
        if (finalizerInitialized) {
            return;
        }

        try {
            finalizerQLength = (LongMonitor) vm.findByName(FINALIZER_QUEUE_LENGTH);
            finalizerQMaxLength = (LongMonitor) vm.findByName(FINALIZER_QUEUE_MAX_LENGTH);
            finalizerTime = (LongMonitor) vm.findByName(FINALIZER_TIME);
            finalizerCount = (LongMonitor) vm.findByName(FINALIZER_OBJECTS);
            if (finalizerQLength == null) {
                return;
            }

            finalizerInitialized = true;
        } catch (MonitorException monitorexception) {
            // Ignore
        }
    }

    private void initialize_post_1_4_1() throws MonitorException {
        newGenMaxSize = (LongMonitor) vm.findByName(NEW_GEN_MAX_SIZE);
        newGenMinSize = (LongMonitor) vm.findByName(NEW_GEN_MIN_SIZE);
        newGenCurSize = (LongMonitor) vm.findByName(NEW_GEN_CURRENT_SIZE);
        lastGCCause = (StringMonitor) vm.findByName(LAST_GC_CAUSE);
        currentGCCause = (StringMonitor) vm.findByName(CURRENT_GC_CAUSE);
        collectorName = (StringMonitor) vm.findByName(NEW_COLLECTOR_NAME);
    }

    private void initialize_common() throws MonitorException {
        survivor0Size = (LongMonitor) vm.findByName(SURVIVOR_FROM_SIZE);
        survivor0Capacity = (LongMonitor) vm.findByName(SURVIVOR_FROM_CAPACITY);
        survivor0Used = (LongMonitor) vm.findByName(SURVIVOR_FROM_USED);

        survivor1Size = (LongMonitor) vm.findByName(SURVIVOR_TO_SIZE);
        survivor1Capacity = (LongMonitor) vm.findByName(SURVIVOR_TO_CAPACITY);
        survivor1Used = (LongMonitor) vm.findByName(SURVIVOR_TO_USED);

        edenSize = (LongMonitor) vm.findByName(EDEN_SIZE);
        edenCapacity = (LongMonitor) vm.findByName(EDEN_CAPACITY);
        edenUsed = (LongMonitor) vm.findByName(EDEN_USED);

        tenuredSize = (LongMonitor) vm.findByName(TENURED_SIZE);
        tenuredCapacity = (LongMonitor) vm.findByName(TENURED_CAPACITY);
        tenuredUsed = (LongMonitor) vm.findByName(TENURED_USED);

        permSize = (LongMonitor) vm.findByName(PERM_SIZE);
        permCapacity = (LongMonitor) vm.findByName(PERM_CAPACITY);
        permUsed = (LongMonitor) vm.findByName(PERM_USED);

        edenGCEvents = (LongMonitor) vm.findByName(EDEN_GC_EVENTS);
        edenGCTime = (LongMonitor) vm.findByName(EDEN_GC_TIME);

        tenuredGCEvents = (LongMonitor) vm.findByName(TENURED_GC_EVENTS);
        tenuredGCTime = (LongMonitor) vm.findByName(TENURED_GC_TIME);

        ageTableSize = (LongMonitor) vm.findByName(AGETABLE_SIZE);

        if (ageTableSize != null) {

            maxTenuringThreshold = (LongMonitor) vm.findByName(MAX_TENURING_THRESHOLD);
            tenuringThreshold = (LongMonitor) vm.findByName(TENURING_THRESHOLD);
            desiredSurvivorSize = (LongMonitor) vm.findByName(DESIRED_SURVIVOR_SIZE);

            int i = (int) ageTableSize.longValue();

            ageTableSizes = new LongMonitor[i];

            for (int j = 0; j < i; j++) {
                if (j < 10) {
                    ageTableSizes[j] = (LongMonitor) vm.findByName(AGETABLE_BYTES_PREFIX + "0" + j);
                } else {
                    ageTableSizes[j] = (LongMonitor) vm.findByName(AGETABLE_BYTES_PREFIX + j);
                }
            }
        }

        classLoadTime = (LongMonitor) vm.findByName(CLASS_LOAD_TIME);
        classesLoaded = (LongMonitor) vm.findByName(CLASSES_LOADED);
        classesUnloaded = (LongMonitor) vm.findByName(CLASSES_UNLOADED);
        classBytesLoaded = (LongMonitor) vm.findByName(CLASS_BYTES_LOADED);
        classBytesUnloaded = (LongMonitor) vm.findByName(CLASS_BYTES_UNLOADED);

        totalCompileTime = (LongMonitor) vm.findByName(TOTAL_COMPILE_TIME);
        totalCompile = (LongMonitor) vm.findByName(TOTAL_COMPILE);

        try {
            initialize_finalizer();
        } catch (Throwable throwable) {
            // Ignore
        }

        osElapsedTime = (LongMonitor) vm.findByName(OS_ELAPSED_TIME);
        osFrequency = (LongMonitor) vm.findByName(OS_FREQUENCY);

        javaCommand = (StringMonitor) vm.findByName(JAVA_COMMAND);
        javaHome = (StringMonitor) vm.findByName(JAVA_HOME);

        vmArgs = (StringMonitor) vm.findByName(VM_ARGS);
        vmFlags = (StringMonitor) vm.findByName(VM_FLAGS);
        vmInfo = (StringMonitor) vm.findByName(VM_INFO);
        vmName = (StringMonitor) vm.findByName(VM_NAME);
        vmVersion = (StringMonitor) vm.findByName(VM_VERSION);
        vmVendor = (StringMonitor) vm.findByName(VM_VENDOR);
        vmSpecName = (StringMonitor) vm.findByName(VM_SPECIFICATION_NAME);
        vmSpecVersion = (StringMonitor) vm.findByName(VM_SPECIFICATION_VERSION);
        vmSpecVendor = (StringMonitor) vm.findByName(VM_SPECIFICATION_VENDOR);

        classPath = (StringMonitor) vm.findByName(CLASS_PATH);
        bootClassPath = (StringMonitor) vm.findByName(BOOT_CLASS_PATH);
        libraryPath = (StringMonitor) vm.findByName(LIBRARY_PATH);
        bootLibraryPath = (StringMonitor) vm.findByName(BOOT_LIBRARY_PATH);

        endorsedDirs = (StringMonitor) vm.findByName(ENDORSED_DIRS);
        extDirs = (StringMonitor) vm.findByName(EXT_DIRS);

        lastModificationTime = (LongMonitor) vm.findByName(LAST_MODIFICATION_TIME);
    }

    void initialize() throws MonitorException {
        initialize_common();

        if (!vmVersion.stringValue().startsWith("1.4.1")) {
            initialize_post_1_4_1();
        }
    }

    public MonitoredVmModel(MonitoredVm monitoredvm) throws MonitorException {
        finalizerInitialized = false;
        vm = monitoredvm;
        initialize();
    }

    public long getNewGenMaxSize() {
        if (newGenMaxSize != null) {
            return newGenMaxSize.longValue();
        } else {
            return edenSize.longValue() + survivor0Size.longValue() + survivor1Size.longValue();
        }
    }

    public long getNewGenMinSize() {
        if (newGenMinSize != null) {
            return newGenMinSize.longValue();
        } else {
            return edenSize.longValue() + survivor0Size.longValue() + survivor1Size.longValue();
        }
    }

    public long getNewGenCurSize() {
        if (newGenCurSize != null) {
            return newGenCurSize.longValue();
        } else {
            return edenSize.longValue() + survivor0Size.longValue() + survivor1Size.longValue();
        }
    }

    public String getLastGCCause() {
        if (lastGCCause == null) {
            return null;
        } else {
            return lastGCCause.stringValue();
        }
    }

    public String getCurrentGCCause() {
        if (currentGCCause == null) {
            return null;
        } else {
            return currentGCCause.stringValue();
        }
    }

    public long getEdenUsed() {
        return edenUsed.longValue();
    }

    public long getTenuredUsed() {
        return tenuredUsed.longValue();
    }

    public long getSurvivor0Used() {
        return survivor0Used.longValue();
    }

    public long getSurvivor1Used() {
        return survivor1Used.longValue();
    }

    public long getPermUsed() {
        return permUsed.longValue();
    }

    public long getEdenSize() {
        return edenSize.longValue();
    }

    public long getSurvivor0Size() {
        return survivor0Size.longValue();
    }

    public long getSurvivor1Size() {
        return survivor1Size.longValue();
    }

    public long getTenuredSize() {
        return tenuredSize.longValue();
    }

    public long getPermSize() {
        return permSize.longValue();
    }

    public long getEdenCapacity() {
        return edenCapacity.longValue();
    }

    public long getSurvivor0Capacity() {
        return survivor0Capacity.longValue();
    }

    public long getSurvivor1Capacity() {
        return survivor1Capacity.longValue();
    }

    public long getTenuredCapacity() {
        return tenuredCapacity.longValue();
    }

    public long getPermCapacity() {
        return permCapacity.longValue();
    }

    public long getEdenGCEvents() {
        return edenGCEvents.longValue();
    }

    public long getTenuredGCEvents() {
        return tenuredGCEvents.longValue();
    }

    public long getEdenGCTime() {
        return edenGCTime.longValue();
    }

    public long getTenuredGCTime() {
        return tenuredGCTime.longValue();
    }

    public long getTenuringThreshold() {
        if (tenuringThreshold == null) {
            return 0L;
        } else {
            return tenuringThreshold.longValue();
        }
    }

    public long getMaxTenuringThreshold() {
        if (maxTenuringThreshold == null) {
            return 0L;
        } else {
            return maxTenuringThreshold.longValue();
        }
    }

    public long getDesiredSurvivorSize() {
        if (desiredSurvivorSize == null) {
            return 0L;
        } else {
            return desiredSurvivorSize.longValue();
        }
    }

    public long getAgeTableSize() {
        if (ageTableSize == null) {
            return 0L;
        } else {
            return ageTableSize.longValue();
        }
    }

    public long[] getAgeTableSizes() {
        if (ageTableSize == null) {
            return null;
        }

        long[] ageTableSizes = new long[this.ageTableSizes.length];

        for (int i = 0; i < this.ageTableSizes.length; i++) {
            ageTableSizes[i] = this.ageTableSizes[i].longValue();
        }

        return ageTableSizes;
    }

    public void getAgeTableSizes(long[] ageTableSizes) {
        if (ageTableSize == null) {
            return;
        }

        for (int i = 0; i < this.ageTableSizes.length; i++) {
            ageTableSizes[i] = this.ageTableSizes[i].longValue();
        }
    }

    public long getClassLoadTime() {
        return classLoadTime.longValue();
    }

    public long getClassesLoaded() {
        return classesLoaded.longValue();
    }

    public long getClassesUnloaded() {
        return classesUnloaded.longValue();
    }

    public long getClassBytesLoaded() {
        return classBytesLoaded.longValue();
    }

    public long getClassBytesUnloaded() {
        return classBytesUnloaded.longValue();
    }

    public long getTotalCompileTime() {
        return totalCompileTime.longValue();
    }

    public long getTotalCompile() {
        return totalCompile.longValue();
    }

    public StringMonitor getCollectorName() {
        return collectorName;
    }

    public boolean isFinalizerInitialized() {
        return finalizerInitialized;
    }

    public void initializeFinalizer() {
        initialize_finalizer();
    }

    public synchronized long getFinalizerTime() {
        if (finalizerInitialized && finalizerTime == null) {
            try {

                finalizerTime = (LongMonitor) vm.findByName(FINALIZER_TIME);
            } catch (MonitorException monitorexception) {
                // Ignore
            }
        }

        if (finalizerTime == null) {
            return 0L;
        }

        return finalizerTime.longValue();
    }

    public synchronized long getFinalizerCount() {
        if (finalizerInitialized && finalizerCount == null) {
            try {

                finalizerCount = (LongMonitor) vm.findByName(FINALIZER_OBJECTS);
            } catch (MonitorException monitorexception) {
                // Ignore
            }
        }

        if (finalizerCount == null) {
            return 0L;
        }
        return finalizerCount.longValue();
    }

    public long getFinalizerQLength() {
        if (finalizerQLength == null) {
            return 0L;
        } else {
            return finalizerQLength.longValue();
        }
    }

    public long getFinalizerQMaxLength() {
        if (finalizerQMaxLength == null) {
            return 0L;
        } else {
            return finalizerQMaxLength.longValue();
        }
    }

    public long getOsElapsedTime() {
        return osElapsedTime.longValue();
    }

    public long getOsFrequency() {
        return osFrequency.longValue();
    }

    public String getJavaCommand() {
        if (javaCommand == null) {
            return null;
        } else {
            return javaCommand.stringValue();
        }
    }

    public String getJavaHome() {
        if (javaHome == null) {
            return null;
        } else {
            return javaHome.stringValue();
        }
    }

    public String getVmArgs() {
        if (vmArgs == null) {
            return null;
        } else {
            return vmArgs.stringValue();
        }
    }

    public String getVmFlags() {
        if (vmFlags == null) {
            return null;
        } else {
            return vmFlags.stringValue();
        }
    }

    public String getClassPath() {
        if (classPath == null) {
            return null;
        } else {
            return classPath.stringValue();
        }
    }

    public String getEndorsedDirs() {
        if (endorsedDirs == null) {
            return null;
        } else {
            return endorsedDirs.stringValue();
        }
    }

    public String getExtDirs() {
        if (extDirs == null) {
            return null;
        } else {
            return extDirs.stringValue();
        }
    }

    public String getLibraryPath() {
        if (libraryPath == null) {
            return null;
        } else {
            return libraryPath.stringValue();
        }
    }

    public String getBootClassPath() {
        if (bootClassPath == null) {
            return null;
        } else {
            return bootClassPath.stringValue();
        }
    }

    public String getBootLibraryPath() {
        if (bootLibraryPath == null) {
            return null;
        } else {
            return bootLibraryPath.stringValue();
        }
    }

    public String getVmInfo() {
        if (vmInfo == null) {
            return null;
        } else {
            return vmInfo.stringValue();
        }
    }

    public String getVmName() {
        if (vmName == null) {
            return null;
        } else {
            return vmName.stringValue();
        }
    }

    public String getVmVersion() {
        if (vmVersion == null) {
            return null;
        } else {
            return vmVersion.stringValue();
        }
    }

    public String getVmVendor() {
        if (vmVendor == null) {
            return null;
        } else {
            return vmVendor.stringValue();
        }
    }

    public String getVmSpecName() {
        if (vmSpecName == null) {
            return null;
        } else {
            return vmSpecName.stringValue();
        }
    }

    public String getVmSpecVersion() {
        if (vmSpecVersion == null) {
            return null;
        } else {
            return vmSpecVersion.stringValue();
        }
    }

    public String getVmSpecVendor() {
        if (vmSpecVendor == null) {
            return null;
        } else {
            return vmSpecVendor.stringValue();
        }
    }

    public long getLastModificationTime() {
        return lastModificationTime.longValue();
    }
}