/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/
package net.sf.intelliplugin.visualgc.ui;

import javax.swing.*;
import java.awt.*;

/**
 * Panel to configure the VMIdentifier.
 * 
 * @author Bart Cremers
 * @since 27-nov-2007
 */
class VMIdentifierPanel extends JPanel {

    private static final String PROTOCOL = "The communications protocol.";
    private static final String LVMID = "";
    private static final String HOST_NAME = "A hostname or IP address indicating the target host.";
    private static final String PORT = "The port for the communications protocol.";
    private static final String SERVER_NAME = "The protocol specific part of the URI.";

    private final JTextField tfProtocol;
    private final JTextField tfLVMId;
    private final JTextField tfHostName;
    private final JTextField tfPort;
    private final JTextField tfServerName;

    public VMIdentifierPanel() {
        super(new GridBagLayout());
        setBorder(BorderFactory.createEmptyBorder(10, 10, 0, 10));

        JLabel lblProtocol = new JLabel("Protocol:");
        tfProtocol = new JTextField(8);
        lblProtocol.setLabelFor(tfProtocol);
        lblProtocol.setDisplayedMnemonic('r');
        lblProtocol.setToolTipText(PROTOCOL);
        tfProtocol.setToolTipText(PROTOCOL);

        JLabel lblLVMId = new JLabel("Local VM Id:");
        tfLVMId = new JTextField(5);
        lblLVMId.setLabelFor(tfLVMId);
        lblLVMId.setDisplayedMnemonic('l');
        lblLVMId.setToolTipText(LVMID);
        tfLVMId.setToolTipText(LVMID);

        JLabel lblHostName = new JLabel("Host Name:");
        tfHostName = new JTextField(30);
        lblHostName.setLabelFor(tfHostName);
        lblHostName.setDisplayedMnemonic('h');
        lblHostName.setToolTipText(HOST_NAME);
        tfHostName.setToolTipText(HOST_NAME);

        JLabel lblPort = new JLabel("Port:");
        tfPort = new JTextField(4);
        lblPort.setLabelFor(tfPort);
        lblPort.setDisplayedMnemonic('p');
        lblPort.setToolTipText(PORT);
        tfPort.setToolTipText(PORT);

        JLabel lblServerName = new JLabel("Server Name:");
        tfServerName = new JTextField(30);
        lblServerName.setLabelFor(tfServerName);
        lblServerName.setDisplayedMnemonic('s');
        lblServerName.setToolTipText(SERVER_NAME);
        tfServerName.setToolTipText(SERVER_NAME);

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.anchor = GridBagConstraints.EAST;
        gbc.insets = new Insets(0, 0, 10, 10);

        gbc.gridy = 0;
        add(lblProtocol, gbc);
        gbc.gridy = 1;
        add(lblLVMId, gbc);
        gbc.gridy = 2;
        add(lblHostName, gbc);
        gbc.gridy = 3;
        add(lblPort, gbc);
        gbc.gridy = 4;
        add(lblServerName, gbc);

        gbc.gridx = 1;
        gbc.insets = new Insets(0, 0, 0, 0);
        gbc.anchor = GridBagConstraints.WEST;

        gbc.gridy = 0;
        add(tfProtocol, gbc);
        gbc.gridy = 1;
        add(tfLVMId, gbc);
        gbc.gridy = 2;
        add(tfHostName, gbc);
        gbc.gridy = 3;
        add(tfPort, gbc);
        gbc.gridy = 4;
        add(tfServerName, gbc);
    }

    public String getVMIdentifier() {
        StringBuilder sb = new StringBuilder();

        String protocol = tfProtocol.getText().trim();
        String lVMId = tfLVMId.getText().trim();
        String hostName = tfHostName.getText().trim();
        String port = tfPort.getText().trim();
        String serverName = tfServerName.getText().trim();

        if (protocol.length() > 0) {
            sb.append(protocol).append("//");
        }
        sb.append(lVMId);
        if (hostName.length() > 0) {
            sb.append("@").append(hostName);
        }
        if (port.length() > 0) {
            sb.append(":").append(port);
        }
        if (serverName.length() > 0) {
            sb.append("/").append(serverName);
        }
        return sb.toString();
    }
}
