/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/
package net.sf.intelliplugin.visualgc.ui;

import com.intellij.openapi.actionSystem.ActionGroup;
import com.intellij.openapi.actionSystem.ActionManager;
import static net.sf.intelliplugin.visualgc.config.IntelliHeapConfiguration.*;
import net.sf.intelliplugin.visualgc.model.GCSample;
import net.sf.intelliplugin.visualgc.util.BorderUtil;

import javax.swing.*;
import java.awt.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * Panel containing all visual information about a JVM.
 *
 * @author Bart Cremers
 * @since 21-nov-2007
 * @see net.sf.intelliplugin.visualgc.ui.RealTime
 * @see net.sf.intelliplugin.visualgc.ui.AgeHistogram
 * @see net.sf.intelliplugin.visualgc.ui.VisualHeap
 */
class VisualGCPanel extends JPanel implements PropertyChangeListener {

    private final RealTime realTime;
    private final AgeHistogram histogram;
    private final VisualHeap heap;

    public VisualGCPanel(GCSample gcSample) {
        super(new BorderLayout());

        histogram = new AgeHistogram(gcSample);
        histogram.setBorder(BorderUtil.createTitledBorder("Histogram", getConfiguration().getBorderColor()));
        histogram.setBackground(getConfiguration().getBackgroundColor());

        heap = new VisualHeap(gcSample);
        heap.setBorder(BorderUtil.createTitledBorder("Heap", getConfiguration().getBorderColor()));
        heap.setBackground(getConfiguration().getBackgroundColor());

        realTime = new RealTime(gcSample);
        realTime.setBorder(BorderUtil.createTitledBorder("Real Time", getConfiguration().getBorderColor()));
        realTime.setBackground(getConfiguration().getBackgroundColor());

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 1.0D;
        gbc.weighty = 1.0D;
        gbc.gridwidth = GridBagConstraints.REMAINDER;

        JPanel centerPane = new JPanel(new GridBagLayout());
        centerPane.setBorder(BorderFactory.createLoweredBevelBorder());

        centerPane.add(heap, gbc);
        centerPane.add(realTime, gbc);

        gbc.gridheight = GridBagConstraints.REMAINDER;
        centerPane.add(histogram, gbc);

        ActionManager actionManager = ActionManager.getInstance();
        ActionGroup actionGroup = (ActionGroup) actionManager.getAction("visualgc.group");
        add(actionManager.createActionToolbar("visualgc.toolbar", actionGroup, true).getComponent(),
            BorderLayout.NORTH);
        add(centerPane, BorderLayout.CENTER);

        getConfiguration().addPropertyChangeListener(this);
    }

    public void update(GCSample sample) {
        histogram.update(sample);
        heap.update(sample);
        realTime.update(sample);
    }

    public void propertyChange(PropertyChangeEvent evt) {
        String pName = evt.getPropertyName();
        if (BACKGROUND_COLOR.equals(pName)) {
            histogram.setBackground(getConfiguration().getBackgroundColor());
            heap.setBackground(getConfiguration().getBackgroundColor());
            realTime.setBackground(getConfiguration().getBackgroundColor());
        } else if (BORDER_COLOR.equals(pName)) {
            BorderUtil.changeTitledBorderColor(histogram, getConfiguration().getBorderColor());
            BorderUtil.changeTitledBorderColor(heap, getConfiguration().getBorderColor());
            BorderUtil.changeTitledBorderColor(realTime, getConfiguration().getBorderColor());
        }
    }
}