/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/
package net.sf.intelliplugin.visualgc.ui.graph;

import java.awt.*;

/**
 * A helper class to paint a grid.
 * 
 * @author Bart Cremers
 * @since 20-nov-2007
 */
class GridDrawer {

    private static final int DEFAULT_X_INCREMENT = 10;
    private static final int DEFAULT_Y_INCREMENT = 10;

    private Color color;
    private Color secundaryColor;
    private double splitLevel;
    private final int xIncrement;
    private final int yIncrement;

    public GridDrawer(int xIncrement, int yIncrement, Color color) {
        this.xIncrement = xIncrement;
        this.yIncrement = yIncrement;
        this.color = color;
    }

    public GridDrawer(Color color) {
        this(DEFAULT_X_INCREMENT, DEFAULT_Y_INCREMENT, color);
    }

    public Color getPrimaryColor() {
        return color;
    }

    public void setPrimaryColor(Color color) {
        this.color = color;
    }

    public Color getSecundaryColor() {
        return secundaryColor;
    }

    public void setSecundaryColor(Color secundaryColor) {
        this.secundaryColor = secundaryColor;
    }

    public void splitRange(double splitLevel, Color secundaryColor) {
        this.secundaryColor = secundaryColor;
        splitRange(splitLevel);
    }

    public void splitRange(double splitLevel) {
        this.splitLevel = splitLevel;
    }

    void draw(Graphics g, int width, int height) {
        int splitPosition = (int) ((double) (width * height) * splitLevel) / width;
        if (splitPosition > height) {
            splitPosition = height;
        }

        if (splitLevel != 0.0D && splitPosition == 0) {
            splitPosition = 1;
        }

        if (splitPosition != 0) {
            g.setColor(secundaryColor);
            for (int x = 0; x < width; x += xIncrement) {
                g.drawLine(x, 0, x, splitPosition - 1);
            }

            g.drawLine(width - 1, 0, width - 1, splitPosition - 1);

            for (int y = 0; y <= splitPosition - 1; y += yIncrement) {
                g.drawLine(0, y, width - 1, y);
            }
        }

        g.setColor(color);

        for (int x = 0; x < width; x += xIncrement) {
            g.drawLine(x, splitPosition, x, height - 1);
        }

        g.drawLine(width - 1, splitPosition, width - 1, height - 1);

        if (splitLevel == 0.0D || splitPosition % yIncrement == 0) {
            g.drawLine(0, splitPosition, width - 1, splitPosition);
        }

        splitPosition = ((splitPosition + yIncrement) / yIncrement) * yIncrement;

        for (int y = splitPosition; y < height; y += yIncrement) {
            g.drawLine(0, y, width - 1, y);
        }

        g.drawLine(0, height - 1, width, height - 1);
    }
}