/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/
package net.sf.intelliplugin.visualgc.ui.graph;

import net.sf.intelliplugin.visualgc.config.IntelliHeapConfiguration;

import javax.swing.*;
import java.awt.*;

/**
 * A component to draw a level on a grid.
 * 
 * @author Bart Cremers
 * @since 20-nov-2007
 */
public class Level extends JComponent {

    private Color color;
    private double level;
    private final GridDrawer gridDrawer;
    private static final Color DEFAULT_COLOR = Color.WHITE;

    public Level(Color color) {
        this.color = color;
        gridDrawer = new GridDrawer(10, 10, IntelliHeapConfiguration.getConfiguration().getPrimaryGridColor());
    }

    public Level() {
        this(DEFAULT_COLOR);
    }

    public void draw() {
        repaint();
    }

    public void update(Graphics g) {
        paint(g);
    }

    public void updateLevel(double level) {
        this.level = level;
    }

    public void updateGrayLevel(double grayLevel) {
        gridDrawer.splitRange(grayLevel, IntelliHeapConfiguration.getConfiguration().getSecundaryGridColor());
    }

    void drawLevel(Graphics g) {
        int height = getHeight();
        int width = getWidth();
        int level = (int) ((double) (width * height) * this.level) / width;
        if (level > height) {
            level = height;
        }

        if (this.level != 0.0D && level == 0) {
            level = 1;
        }

        g.setColor(color);

        g.fill3DRect(0, height - level, width, height, true);
    }

    public Color getColor() {
        return color;
    }

    public void setColor(Color color) {
        this.color = color;
        repaint();
    }

    public void setGridColors(Color primaryColor, Color secundaryColor) {
        gridDrawer.setPrimaryColor(primaryColor);
        gridDrawer.setSecundaryColor(secundaryColor);
    }

    public void paint(Graphics g) {
        gridDrawer.draw(g, getWidth(), getHeight());
        drawLevel(g);
        super.paint(g);
    }
}