/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/
package net.sf.intelliplugin.visualgc.util;

import com.intellij.openapi.application.ApplicationManager;

import javax.swing.*;
import java.awt.*;
import java.awt.event.WindowListener;

import net.sf.intelliplugin.visualgc.ui.IntelliHeapComponent;

/**
 * A helper class for displaying a dialog.
 * 
 * @author Bart Cremers
 * @since 27-nov-2007
 */
public class DialogHelper {
    private DialogHelper() {}
    
    public static void showInDialog(String title, JPanel dialogPanel, JPanel buttonPanel) {
        showInDialog(title, dialogPanel, buttonPanel, null);
    }

    public static void showInDialog(String title, JPanel dialogPanel, JPanel buttonPanel,
                                    WindowListener customWindowListener) {

        IntelliHeapComponent component = (IntelliHeapComponent) ApplicationManager.getApplication().getComponent(
            IntelliHeapComponent.COMPONENT_NAME);
        if (component != null) {
            JDialog dialog = new JDialog(SwingUtilities.getWindowAncestor(component.getContentPane()), title,
                                         Dialog.ModalityType.APPLICATION_MODAL);
            dialog.add(dialogPanel, BorderLayout.CENTER);
            dialog.add(buttonPanel, BorderLayout.SOUTH);

            if (customWindowListener != null) {
                dialog.addWindowListener(customWindowListener);
            }
            
            dialog.pack();
            dialog.setLocationRelativeTo(component.getContentPane());
            dialog.setVisible(true);
        }
    }
}
