/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/
package net.sf.intelliplugin.visualgc.util;

import com.intellij.openapi.ui.LabeledComponent;

import javax.swing.*;
import java.awt.*;

/**
 * Helper class for the GridBagLayout.
 * 
 * @author Bart Cremers
 * @since 3-dec-2007
 */
public class GridBagUtil {

    private GridBagUtil() {
    }

    public static void createVerticalForm(JPanel panel, String[] labels, JComponent[] components) {
        panel.setLayout(new GridBagLayout());

        GridBagConstraints gbc = new GridBagConstraints();

        int offset = 0;
        for (int i = 0; i < components.length; i++) {
            gbc.gridy = i + offset;
            if (components[i] == null) {
                gbc.gridwidth = 2;

                panel.add(Box.createVerticalStrut(20), gbc);

                offset++;    
            } else {
                LabeledComponent.TextWithMnemonic textWithMnemonic =
                    LabeledComponent.TextWithMnemonic.fromTextWithMnemonic(labels[i - offset]);

                gbc.gridwidth = 1;

                JLabel label = new JLabel();
                textWithMnemonic.setToLabel(label);
                label.setLabelFor(components[i]);

                gbc.gridx = 0;
                gbc.anchor = GridBagConstraints.EAST;

                panel.add(label, gbc);

                gbc.gridx = 1;
                gbc.anchor = GridBagConstraints.WEST;

                panel.add(components[i], gbc);
            }
        }
    }
}
